import os
import shutil
import logging
from graftm.graftm_package import GraftMPackage
from graftm.prerequisite_checker import PrerequisiteChecker
import inspect

PIPELINE_AA = "P"
PIPELINE_NT = "D"

class InvalidFileExtensionError(Exception):
    pass

class UninstalledProgramError(Exception):
    pass
    
class HouseKeeping:
    ### Functions for setting up the graftM pipeline to run correctly, and 
    ### general housekeeping things like adding and removing directories and 
    ### files.

    def file_basename(self, file):
        '''
        Strips the path and last extension from the file variable. If the 
        extension is found to be valid, the basename will be returned. Otherwise
        an error will be raise and graftM will exit
        '''
        valid_extensions = set(".tree",
                               ".tre")
        split_file = os.path.basename(file).split('.')
        base, suffix = '.'.join(split_file[:-1]), split_file[-1]
        
        if suffix in valid_extensions:
            return base
        else:
            logging.error("Invalid file extension found on file: %s" % file)
            logging.error("For trees, please provide a file with one of the \
following extensions: %s" % ' '.join(valid_extensions.keys()))
            raise InvalidFileExtensionError
            
            
    def set_euk_hmm(self, args):
        'Set the hmm used by graftM to cross check for euks.'
        if hasattr(args, 'euk_hmm_file'):
            pass
        elif not hasattr(args, 'euk_hmm_file'):
            # set to path based on the location of bin/graftM, which has
            # a more stable relative path to the HMM when installed through
            # pip.
            setattr(args, 'euk_hmm_file', os.path.join(os.path.dirname(inspect.stack()[-1][1]),'..','share', '18S.hmm'))
        else:
            raise Exception('Programming Error: setting the euk HMM')    
    
    def setpipe(self, hmm):
        ## Read in the hmm file and return the evalue trusted cutoff and the
        ## type of HMM
        hmm_tc = None
        hmm_read = [x.rstrip() for x in open(hmm, 'r').readlines() if x.startswith('ALPH') or x.startswith('TC')]
        if len(hmm_read) > 2:
            raise Exception('Possibly Programming Error: Misread HMM library.')
        for item in hmm_read:
            if item.startswith('ALPH'):
                if item.split()[1] == 'DNA' or item.split()[1] == 'RNA':
                    hmm_type = 'D'
                elif item.split()[1] == 'amino':
                    hmm_type = 'P'
            elif item.startswith('TC'):
                hmm_tc = item.split()[1]
            else:
                raise Exception('Possibly Programming Error: Misread HMM library.')
        return hmm_type, hmm_tc
        
    def delete(self, delete_list):
        for item in delete_list:
            try:
                os.remove(item)
            except:
                pass

    def make_working_directory(self, directory_path, force):
        if force:
            shutil.rmtree(directory_path, ignore_errors=True)
            os.mkdir(directory_path)
        else:
            try:
                os.mkdir(directory_path)
            except:
                logging.error('Directory %s already exists. Exiting to prevent over-writing' % directory_path)
                raise Exception('Directory %s already exists. Exiting to prevent over-writing'% directory_path)
    
    def parameter_checks(self, args):
        ## Check that the necessary files are in place
        if args.subparser_name == 'graft':
            # Check that the placement cutoff is between 0.5 and 1
            if float(args.placements_cutoff) < float(0.5) or float(args.placements_cutoff) > float(1.0):
                logging.info('Please specify a confidence level (-d) between 0.5 and 1.0! Found: %s' % args.placements_cutoff)
                exit(1)
            
            self._check_file_existence(args.forward)
            if args.reverse:
                self._check_file_existence(args.reverse)
                    
            # Determine the File format based on the suffix
            
            sequence_file_list = []
            if args.reverse:
                if len(args.forward) != len(args.reverse):
                    logging.error('Confusing input. There appears to be different numbers of forward and reverse files specified')
                for i, forward_file in enumerate(args.forward):
                    sequence_file_list.append([forward_file, args.reverse[i]])
            else:
                sequence_file_list = [[f] for f in args.forward]
                
            return sequence_file_list
        
    def _check_file_existence(self, files):
        '''Iterate through files and exit(1) if any do not pass
        os.path.isfile'''
        for f in files:
            if not os.path.isfile(f):
                logging.error("The file '%s' does not appear to exist, stopping" % f)
                exit(1)
            
    
    def checkCreatePrerequisites(self):
        PrerequisiteChecker.check_prerequisites(('taxit','FastTreeMP','seqmagick','hmmalign','mafft'))

    def get_maximum_range(self, hmm):
        '''
        If no maximum range has been specified, and if using a hmm search, a 
        maximum range can be determined by using the length of the HMM 
        
        Parameters
        ----------
        hmm : str
            path to hmm profile
        
        Returns
        -------
        Length to search to when linking hits on a single contig
        '''
        length=int([x for x in open(hmm) if x.startswith("LENG")][0].split()[1])
        max_length=round(length*1.5, 0)
        
        return max_length

    def set_attributes(self, args):
        PrerequisiteChecker.check_prerequisites(('orfm','nhmmer','hmmsearch','fxtract','pplacer','seqmagick','ktImportText','diamond'))
        
        # Read graftM package and assign HMM and refpkg file
        if args.graftm_package:
            if not os.path.isdir(args.graftm_package):
                raise Exception("%s does not exist. Are you sure you provided the correct path?" % args.graftm_package)
            else:
                gpkg = GraftMPackage.acquire(args.graftm_package)
                if hasattr(args, 'search_hmm_files'): # If a hmm is specified, overwrite the one graftM package
                    setattr(args, 'aln_hmm_file', gpkg.alignment_hmm_path())
                    setattr(args, 'reference_package', gpkg.reference_package_path())
                else:
                    setattr(args, 'search_hmm_files', [])
                    for hmm in gpkg.search_hmm_paths():
                        args.search_hmm_files.append(hmm)
                    setattr(args, 'aln_hmm_file', gpkg.alignment_hmm_path())
                    setattr(args, 'reference_package', gpkg.reference_package_path())

        elif hasattr(args, 'search_diamond_files'):
            if args.search_method == 'diamond': 
                if hasattr(args, 'aln_hmm_file'):
                    pass
                else:
                    raise Exception("aln_hmm_file not specified")
            else:
                raise Exception("Specified HMM databases when not using the diamond search pipeline. Using: %s" % (args.search_method))   
        
        elif hasattr(args, 'search_hmm_files'):
            if args.search_method == 'hmmsearch':
                if not hasattr(args, 'aln_hmm_file'):
                    if len(args.search_hmm_files) == 1:
                        if not args.search_only:
                            setattr(args, 'aln_hmm_file', args.search_hmm_files[0])
                    else:
                        raise Exception("Multiple search HMMs specified, but aln_hmm_file not specified")

            else:
                raise Exception("Specified HMM search_hmm_files when not using the hmmsearch pipeline. Using: %s" % (args.search_method))

        elif hasattr(args, 'search_hmm_list_file'):
            if args.search_method == 'hmmsearch':
                setattr(args, 'search_hmm_files', [x.rstrip() for x in open(args.search_hmm_list_file).readlines()])
                if not hasattr(args, 'aln_hmm_file'):
                    if not args.search_only:
                        raise Exception("Multiple search HMMs specified, but aln_hmm_file not specified")
            else:
                raise Exception("Specified HMM search_hmm_files when not using the hmmsearch pipeline. Using: %s" % (args.search_method))
        
        else:
            raise Exception('No refpkg or HMM specified: Do not know what to search with.')




class TaxonomyExtractor:
    def taxonomy_from_annotated_tree(self, tree_node):
        '''Given an annotated tree, return the taxonomy of each tip
        
        Parameters
        ---------
        tree_node: skbio.tree.TreeNode
        
        Returns
        -------
        dictionary of tip name to array of taxonomic affiliations'''
        tip_to_taxonomy = {}
        for tip in tree_node.tips():
            tax = []
            n = tip.parent
            while n:
                if n.name and n.parent:
                    tax = [n.name]+tax
                n = n.parent
            tip_name = tip.name.replace(' ','_')
            if tip_name in tip_to_taxonomy:
                raise Exception("Found duplicate tip name '%s'" % tip_name)
            if len(tax)==0:
                tip_to_taxonomy[tip_name] = []
            else:
                tip_to_taxonomy[tip_name] = [t.strip() for t in '; '.join(tax).split(';')]
        return tip_to_taxonomy


#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import tempfile
import os
import re
import sys

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
import graftm.hmmsearcher

class HmmsearcherTests(unittest.TestCase):
    path_to_data = os.path.join(os.path.dirname(os.path.realpath(__file__)),'data')
    
    def test_generates_correct_cmd_single_hmm(self):
        searcher = graftm.hmmsearcher.HmmSearcher(1)
        cmd = searcher._hmm_command('orfm some', [(['hmm1','out1'],1)])
        self.assertEqual('orfm some | hmmsearch  --cpu 1 -o /dev/null --noali --domtblout out1 hmm1 -', cmd)
        
    def test_generates_correct_cmd_extra_args(self):
        searcher = graftm.hmmsearcher.HmmSearcher(1, extra_args='-E 1e-99')
        cmd = searcher._hmm_command('orfm some', [(['hmm1','out1'],1)])
        self.assertEqual('orfm some | hmmsearch -E 1e-99 --cpu 1 -o /dev/null --noali --domtblout out1 hmm1 -', cmd)
        
    def test_generates_multi_hmms(self):
        searcher = graftm.hmmsearcher.HmmSearcher(1)
        cmd = searcher._hmm_command('orfm some', [(['hmm1','out1'],1), (['hmm2','out2'],2)])
        self.assertEqual('orfm some | tee >(hmmsearch  --cpu 1 -o /dev/null --noali --domtblout out1 hmm1 -) | hmmsearch  --cpu 2 -o /dev/null --noali --domtblout out2 hmm2 -', cmd)
        
    def test_munch_off_batch_single_cpu(self):
        searcher = graftm.hmmsearcher.HmmSearcher(1)
        queue = [['hmm1','out1'],['hmm2','out2']]
        pairs_to_run = searcher._munch_off_batch(queue)
        self.assertEqual([[['hmm1','out1'],1]], pairs_to_run)
        pairs_to_run = searcher._munch_off_batch(queue)
        self.assertEqual([[['hmm2','out2'],1]], pairs_to_run)
        
    def test_munch_off_batch_single_hmm(self):
        searcher = graftm.hmmsearcher.HmmSearcher(10)
        queue = [['hmm1','out1']]
        pairs_to_run = searcher._munch_off_batch(queue)
        self.assertEqual([[['hmm1','out1'],10]], pairs_to_run)
        
    def test_munch_off_batch_multi_cpu(self):
        searcher = graftm.hmmsearcher.HmmSearcher(2)
        queue = [['hmm1','out1'],['hmm2','out2'],['hmm3','out3']]
        pairs_to_run = searcher._munch_off_batch(queue)
        self.assertEqual([[['hmm1','out1'],1], [['hmm2','out2'],1]], pairs_to_run)
        pairs_to_run = searcher._munch_off_batch(queue)
        self.assertEqual([[['hmm3','out3'],2]], pairs_to_run)
        
    def test_munch_off_batch_multi_cpu_heterogenous(self):
        searcher = graftm.hmmsearcher.HmmSearcher(5)
        queue = [['hmm1','out1'],['hmm2','out2']]
        pairs_to_run = searcher._munch_off_batch(queue)
        self.assertEqual([[['hmm1','out1'],3], [['hmm2','out2'],2]], pairs_to_run)
        
    def test_actually_runs(self):
        searcher = graftm.hmmsearcher.HmmSearcher(5)
        faa_file = os.path.join(self.path_to_data, 'mcrA.gpkg/mcrA_1.1.faa')
        hmm_file = os.path.join(self.path_to_data, 'mcrA.gpkg/mcrA.hmm')
        with tempfile.NamedTemporaryFile(suffix='.fa') as output:
            searcher.hmmsearch('cat %s' % faa_file, 
                               [hmm_file],
                               [output.name])
            expected = '''#                                                                             --- full sequence --- -------------- this domain -------------   hmm coord   ali coord   env coord
# target name         accession   tlen query name           accession   qlen   E-value  score  bias   #  of  c-Evalue  i-Evalue  score  bias  from    to  from    to  from    to  acc description of target
# ------------------- ---------- ----- -------------------- ---------- ----- --------- ------ ----- --- --- --------- --------- ------ ----- ----- ----- ----- ----- ----- ----- ---- ---------------------
example_partial_mcra8 -            162 mcrA.fasta           -            557     2e-88  286.6   5.0   1   1   2.5e-89   2.2e-88  286.4   5.0   332   487     1   162     1   162 0.99 -
#'''.split('\n')
            observed = re.sub(r'\t', ' ', open(output.name).read()).split("\n")
            self.assertEqual(expected, observed[:5])
        
    def test_nhmmer_searcher(self):
        searcher = graftm.hmmsearcher.NhmmerSearcher(20)
        cmd = searcher._hmm_command('cat some', [(['hmm1','out1'],10)])
        self.assertEqual('cat some | nhmmer  --cpu 10 -o /dev/null --noali --tblout out1 hmm1 -', cmd)        
        


if __name__ == "__main__":
    unittest.main()



import logging
import re

class OrfM:
    def __init__(self, **kwargs):
        self.min_orf_length = kwargs.pop('min_orf_length',96)
        self.restrict_read_length = kwargs.pop('restrict_read_length',None)
        if len(kwargs) > 0:
            raise Exception("Unexpected arguments detected: %s" % kwargs)
        
    def command_line(self, input_path=None):
        '''Return a string to run OrfM with, assuming sequences are incoming on
        stdin and printed to stdout
        
        Parameters
        ----------
        input_path: str
            path to the input path, or None for STDIN being the input
        '''
        
        if self.min_orf_length:
            orfm_arg_l = " -m %d" % self.min_orf_length
        else:
            orfm_arg_l = ''
        
        if self.restrict_read_length:
            orfm_arg_l = " -l %d" % self.restrict_read_length
            
        cmd = 'orfm %s ' % orfm_arg_l
        if input_path:
            cmd += input_path
        logging.debug("OrfM command chunk: %s" % cmd)
        return cmd
    
    @staticmethod
    def regular_expression():
        '''Return a compiled regular expression matching the OrfM output defline
        with the originalName, startPosition, frameNumber, orfNumber being
        the contents of the regular expression'''
        return re.compile('^(\S+)_(\d+)_(\d)_(\d+)')

class ZcatOrfM(OrfM):
    '''Like OrfM, except separate the zcat and the orfm commands into two
    parts of a shell pipeline'''
    
    def command_line(self, input_path=None):
        if input_path is None:
            raise Exception("input_path required for ZcatOrfM")
        original = OrfM.command_line(self, input_path=None)
        return "zcat '%s' | %s" % (input_path, original)


import logging
from graftm.sequence_search_results import SequenceSearchResult

class SearchTableWriter:
    '''
    Class for writing the search output OTU table. Basically a summary
    of hits to the HMM/Diamond searched in the following format:
    
             #ID    Metagenome_1    Metagenome_2    ...
            HMM1    50              6
            HMM2    195             41
            HMM3    2               20120
            ...
            
    You just need to specify a series of SequenceSearchResult objects, and an
    output path.
    '''
    
    def _interpret_hits(self, results_list, base_list):
        '''Sort reads that hit multiple HMMs to the databases to which they had
        the highest bit score. Return a dictionary containing HMMs as keys, and 
        number of hits as the values.
        
        This function is set up so that the read names could easily be returned
        instead of numbers, for future development of GraftM
        
        Parameters
        ----------
        results_list: list
            Iterable if SequenceSearchResult objects. e.g.
                [SequenceSearchResult_1, SequenceSearchResult_2, ...]
        
        base_list: list
            Iterable of the basenames for each sequence file provided to graftM
            e.g.
                [sample_1, sample_2, ...]
                
        Returns
        -------
        dictionary:
            Contains samples as entries. The value for each sample is another 
            dictionary with HMM as the key, and number of hits as values:
                {"sample_1":{HMM_1: 12
                             HMM_2: 35
                             HMM_3: 1258
                             ...}
                 ...
                }
            
        '''
        logging.debug("Sorting reads into HMMs by bit score")
        
        run_results = {}

        ########################################################################
        ################## - Sort reads to best hit db - #######################
        for base, results in zip(base_list, results_list): # For each sample
            search_results = {}
            for search in results():
                search_list = list(
                                   search.each([SequenceSearchResult.QUERY_ID_FIELD,
                                                SequenceSearchResult.ALIGNMENT_BIT_SCORE,
                                                SequenceSearchResult.HMM_NAME_FIELD])
                                   )
                for hit in search_list:
                    if hit[0] in search_results:
                        if float(hit[1]) > search_results[hit[0]][1]:
                            search_results[hit[0]] = [float(hit[1]), hit[2]]
                    else:
                        search_results[hit[0]] = [float(hit[1]), hit[2]]
            run_results[base] = search_results
        
        ########################################################################
        ################## - Gather counts for each db - #######################
        db_count = {}
        for run in run_results.keys():
            run_count = {}
            for entry in run_results[run].values():
                key = entry[1]
                if key in run_count:
                    run_count[key] += 1
                else:
                    run_count[key] = 1
            db_count[run] = run_count
        
        return db_count    
    
    def _write_results(self, db_count, output_path):
        '''Write the table to the output_path directory
        
        db_count: dict
            Contains samples as entries. The value for each sample is another 
            dictionary with HMM as the key, and number of hits as values:
                {"sample_1":{HMM_1: 12
                             HMM_2: 35
                             HMM_3: 1258
                             ...}
                 ...
                }
        
        output_path: str
            Path to output file to which the resultant output file will be 
            written to.
        '''
        
        logging.debug("Writing search otu table to file: %s" % output_path)
        
        output_dict = {}
        
        for idx, value_dict in enumerate(db_count.values()):
            for database, count in value_dict.iteritems():
                if database in output_dict:
                    output_dict[database].append(str(count))
                else:
                    output_dict[database] = ['0']*idx + [str(count)]
            
            for key, item in output_dict.iteritems():
                if len(item) == idx:
                    output_dict[key].append('0')
        
        with open(output_path, 'w') as out:
            out.write('\t'.join(["#ID"] + db_count.keys()) + '\n')
            for key, item in output_dict.iteritems():
                out.write("%s\t%s" % (key, '\t'.join(item)) + '\n' )
        
    def build_search_otu_table(self, search_results_list, base_list, output_path):
        '''
        Build an OTU from SequenceSearchResult objects
        
        Parameters
        ----------
        search_results_list: list
            Iterable if SequenceSearchResult objects. e.g.
                [SequenceSearchResult_1, SequenceSearchResult_2, ...]
        base_list: list
            Iterable of the basenames for each sequence file provided to graftM
            e.g.
                [sample_1, sample_2, ...]
        output_path: str
            Path to output file to which the resultant output file will be 
            written to.
        '''
        
        db_count = self._interpret_hits(search_results_list,
                                        base_list)
        
        self._write_results(db_count, output_path)
        
        
        



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################
#
#
#
###############################################################################
################################ - Imports - ##################################
# Local

# System
import itertools
import heapq
from skbio import TreeNode


################################ - Statics - ##################################

################################ - Classes - ##################################

class BinaryTreeError(Exception):
    pass

class NoGoodRootError(Exception):
    pass

################################# - Code - ####################################

class Rerooter:
    def __init__(self): pass
    
    def reroot(self, tree):
        '''
        Reroot a tree: This small piece of code is intended to take a rooted 
        tree that is NOT binary, that is to say, the root node has 3 or more
        child nodes. To create a binary tree, this function will find the two 
        longest distances between the child nodes, find which child node is 
        common to them both, and reroot at the branch connecting that node 
        to root. For example, the new root will be placed where the "x" is in 
        the following example tree:
                                      _
                                   __|_     _
                                  |_____x__|
                                --|__ _    |_
                                     |_
        
        Parameters
        ----------
        tree: TreeNode object
            TreeNode object (tree opened with skbio)
        '''   
        children = tree.children
        no_found_root = True
        branches_to_combine = 2
        if len(children) > 2:
            node_combinations = list(itertools\
                                   .combinations([x for x in children], 
                                                 2))
            length_combination = list(itertools\
                                   .combinations([x.length for x in children], 
                                                 2))
            sums = [x[0] + x[1] for x in length_combination]
            
            while no_found_root:
                
                nodes = [node_combinations[sums.index(x)]
                         for x in heapq.nlargest(2, sums)]
                most_distant_node = [node for node in nodes[0] 
                                     if node in nodes[1]]
                if len(most_distant_node) > 0:
                    most_distant_node = most_distant_node[0]
                    no_found_root = False
                
            other_nodes = [node for node in children 
                           if node != most_distant_node]
            link_root = TreeNode(children=other_nodes)
            tree = TreeNode(children=[link_root,tree])
            return tree
        else:
            raise BinaryTreeError("Tree is already binary. Something has \
mislead me here!")

###############################################################################
###############################################################################



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

exec(open('graftm/version.py').read()) # loads __version__

setup(name='graftm',
      version=__version__,
      author='Joel Boyd, Ben Woodcroft',
      description='GraftM is a pipeline used for identifying and classifying marker gene reads from metagenomic datasets',
      long_description=readme,
      license='GPL3+',
      keywords="",
      packages=find_packages(exclude='docs'),
      install_requires=('biopython >=1.64',
                        'seqmagick >=0.5.0',
                        'scikit-bio >=0.2.2',
                        'subprocess32 >=3.2.6',
                        'biom-format >=2.1.4',
                        'extern >=0.0.4',
                        'taxtastic >=0.5.4',
                        'tempdir >=0.6'),
      url='http://geronimp.github.io/graftM',
      scripts=['bin/graftM'],
      data_files=[
          ('share', ['share/18S.hmm']),
      ],
)



import json
import os
import shutil
import logging

class InsufficientGraftMPackageException(Exception): pass

class GraftMPackage:
    '''Package to represent a GraftM package. To start using a package, run

    pkg = GraftMPackage.acquire('/path/to/a.gpkg')

    and then retrieve values with e.g.

    pkg.alignment_hmm_path()
      #=> '/path/to/a.gpkg/the.hmm'
    '''

    _CONTENTS_FILE_NAME = 'CONTENTS.json'

    # The key names are unlikely to change across package format versions,
    # so store them here in the superclass
    DIAMOND_DATABASE_KEY = 'diamond_database'
    VERSION_KEY = 'graftm_package_version'
    ALIGNMENT_HMM_KEY = 'align_hmm'
    SEARCH_HMM_KEY = "search_hmms"
    REFERENCE_PACKAGE_KEY = "refpkg"
    HMM_TRUSTED_CUTOFF_KEY = "trusted_cutoff"
    RANGE_KEY = "range"
    UNALIGNED_SEQUENCE_DATABASE_KEY = "unaligned_sequence_database"
    _CONTENTS_FILE_NAME = 'CONTENTS.json'

    _CURRENT_VERSION = 3

    _REQUIRED_KEYS = {'2': [
                             VERSION_KEY,
                             ALIGNMENT_HMM_KEY,
                             SEARCH_HMM_KEY,
                             REFERENCE_PACKAGE_KEY,
                             HMM_TRUSTED_CUTOFF_KEY,
                             RANGE_KEY
                             ],
                      '3': [
                             VERSION_KEY,
                             ALIGNMENT_HMM_KEY,
                             SEARCH_HMM_KEY,
                             REFERENCE_PACKAGE_KEY,
                             HMM_TRUSTED_CUTOFF_KEY,
                             RANGE_KEY,
                             UNALIGNED_SEQUENCE_DATABASE_KEY
                             ]
                      }


    @staticmethod
    def acquire(graftm_package_path):
        '''Acquire a new graftm Package

        Parameters
        ----------
        graftm_output_path: str
            path to base directory of graftm
        '''
        
        contents_hash = json.load(
                                   open(
                                        os.path.join(
                                                     graftm_package_path,
                                                     GraftMPackage._CONTENTS_FILE_NAME
                                                     ),
                                         )
                                   )
        
        
        v=contents_hash[GraftMPackage.VERSION_KEY]
        logging.debug("Loading version %i GraftM package: %s" % (v, graftm_package_path))
        if v == 2:
            pkg = GraftMPackageVersion2()
        elif v == 3:
            pkg = GraftMPackageVersion3()
        else:
            raise InsufficientGraftMPackageException("Bad version: %s" % v)
        
        pkg._contents_hash = contents_hash
        pkg._base_directory = graftm_package_path
        # check we are at current version otherwise choke
        pkg.check_universal_keys(v)
        pkg.check_required_keys(GraftMPackage._REQUIRED_KEYS[str(v)])
        return pkg

    def check_universal_keys(self, version):
        h = self._contents_hash
        try:
            v = h[GraftMPackage.VERSION_KEY]
        except KeyError:
            raise InsufficientGraftMPackageException("No version information in graftm package")
        if v != version:
            raise InsufficientGraftMPackageException("Bad version: %s" % v)

    def check_required_keys(self, required_keys):
        '''raise InsufficientGraftMPackageException if this package does not
        conform to the standard of the given package'''
        h = self._contents_hash
        for key in required_keys:
            if key not in h:
                raise InsufficientGraftMPackageException("Package missing key %s" % key)

    def __getitem__(self, key):
        '''Return the value of the given key from the contents file'''
        return self.contents_hash[key]

class GraftMPackageVersion2(GraftMPackage):
    version = 2

    def diamond_database_path(self):
        if self._contents_hash[GraftMPackage.DIAMOND_DATABASE_KEY]:
            return os.path.join(self._base_directory,
                                self._contents_hash[GraftMPackage.DIAMOND_DATABASE_KEY])
        else:
            return None
        
    def search_hmm_paths(self):
        return [os.path.join(self._base_directory, x) for x in
                self._contents_hash[GraftMPackage.SEARCH_HMM_KEY]]

    def alignment_hmm_path(self):
        return os.path.join(self._base_directory,
                            self._contents_hash[GraftMPackage.ALIGNMENT_HMM_KEY])

    def reference_package_path(self):
        return os.path.join(self._base_directory,
                            self._contents_hash[GraftMPackage.REFERENCE_PACKAGE_KEY])

    def use_hmm_trusted_cutoff(self):
        return self._contents_hash[GraftMPackage.HMM_TRUSTED_CUTOFF_KEY]

    def maximum_range(self):
        return self._contents_hash[GraftMPackage.RANGE_KEY]

    def _refpkg_contents(self):
        return json.loads(open(os.path.join(self.reference_package_path(), 'CONTENTS.json')).read())

    def taxtastic_seqinfo_path(self):
        return os.path.join(self.reference_package_path(),
                            self._refpkg_contents()['files']['seq_info'])

    def taxtastic_taxonomy_path(self):
        return os.path.join(self.reference_package_path(),
                            self._refpkg_contents()['files']['taxonomy'])
        
    def reference_package_tree_path(self):
        return os.path.join(self.reference_package_path(),
                            self._refpkg_contents()['files']['tree'])

    @staticmethod
    def compile(output_package_path, refpkg_path, hmm_path, diamond_database_file, max_range, 
                trusted_cutoff=False, search_hmm_files=None):
        '''Create a new GraftM package with the given inputs. Any files
        specified as parameters are copied into the final package so can
        be removed after calling this function.

        Parameters
        ----------
        output_package_path: str
            path to the package being created (must not exist)
        refpkg_path: str
            path to pplacer reference package
        hmm_path: str
            path to the align HMM. Used as the search HMM if search_hmm_files
            is None
        diamond_database_file: str
            path to diamond DB file, or None for nucleotide packages
        max_rage: str
            as per maximum_range()
        trusted_cutoff: boolean
            set TC in search HMM

        search_hmm_files: list of str or None
            use these HMMs for search instead of the hmm_path. All
            basenames of these paths must be unique, and not the same as
            hmm_path.
            
        Returns
        -------
        Nothing
        '''

        if os.path.exists(output_package_path):
            raise Exception("Not writing new GraftM package to already existing file/directory with name %s" % output_package_path)
        os.mkdir(output_package_path)

        hmm_file_in_gpkg = os.path.basename(hmm_path)
        shutil.copyfile(hmm_path, os.path.join(output_package_path, hmm_file_in_gpkg))

        if diamond_database_file:
            diamond_database_file_in_gpkg = os.path.basename(diamond_database_file)
            shutil.copyfile(diamond_database_file, os.path.join(output_package_path, diamond_database_file_in_gpkg))

        refpkg_in_gpkg = os.path.basename(refpkg_path)
        shutil.copytree(refpkg_path, os.path.join(output_package_path, refpkg_in_gpkg))

        
        if search_hmm_files:
            search_hmm_files_in_gpkg_names =\
                [os.path.basename(path) for path in search_hmm_files]
            if hmm_file_in_gpkg in search_hmm_files_in_gpkg_names:
                raise Exception("Search and align HMMs must all have different basenames to create a new gpkg")
            if len(set(search_hmm_files_in_gpkg_names)) != len(search_hmm_files_in_gpkg_names):
                raise Exception("Search HMMs must have different basenames to create a new gpkg")
            for i, search_hmm in enumerate(search_hmm_files):
                shutil.copyfile(search_hmm, os.path.join(output_package_path, search_hmm_files_in_gpkg_names[i]))
        else:
            search_hmm_files_in_gpkg_names = [hmm_file_in_gpkg]
        
        contents = {GraftMPackage.VERSION_KEY: GraftMPackageVersion2.version,
                    GraftMPackage.ALIGNMENT_HMM_KEY: hmm_file_in_gpkg,
                    GraftMPackage.SEARCH_HMM_KEY: search_hmm_files_in_gpkg_names,
                    GraftMPackage.REFERENCE_PACKAGE_KEY: refpkg_in_gpkg,
                    GraftMPackage.HMM_TRUSTED_CUTOFF_KEY: trusted_cutoff,
                    GraftMPackage.RANGE_KEY: max_range}
        if diamond_database_file:
            contents[GraftMPackage.DIAMOND_DATABASE_KEY] = diamond_database_file_in_gpkg

        json.dump(contents, open(os.path.join(output_package_path, GraftMPackage._CONTENTS_FILE_NAME), 'w'))

    
class GraftMPackageVersion3(GraftMPackageVersion2):
    
    version = 3
    
    def unaligned_sequence_database_path(self):
        if self._contents_hash[GraftMPackage.UNALIGNED_SEQUENCE_DATABASE_KEY]:
            return os.path.join(self._base_directory,
                                self._contents_hash[GraftMPackage.UNALIGNED_SEQUENCE_DATABASE_KEY])
        else:
            return None
        
    @staticmethod
    def compile(output_package_path, refpkg_path, hmm_path, diamond_database_file, 
                max_range, unaligned_sequence_database=None, 
                trusted_cutoff=False, search_hmm_files=None):
        '''Create a new GraftM package with the given inputs. Any files
        specified as parameters are copied into the final package so can
        be removed after calling this function.
        
        Parameters
        ----------
        output_package_path: str
            path to the package being created (must not exist)
        refpkg_path: str
            path to pplacer reference package
        hmm_path: str
            path to the search and align HMM
        diamond_database_file: str
            path to diamond DB file, or None for nucleotide packages
        max_rage: str
            as per maximum_range()
        unaligned_sequence_database: str
            path to unaligned sequence database
        trusted_cutoff: boolean
            set TC in search HMM
        search_hmm_files: list of str or None
            use these HMMs for search instead of the hmm_path. All
            basenames of these paths must be unique, and not the same as
            hmm_path.
            
        Returns
        -------
        Nothing
        '''
        
        if os.path.exists(output_package_path): 
            raise Exception("Not writing new GraftM package to already existing file/directory with name %s" % output_package_path)
        os.mkdir(output_package_path)
        
        hmm_file_in_gpkg = os.path.basename(hmm_path)
        shutil.copyfile(hmm_path, os.path.join(output_package_path, hmm_file_in_gpkg))
        # Copy unaligned sequence database into graftm package
        if unaligned_sequence_database:
            unaligned_sequence_database_in_gpkg = os.path.join(output_package_path, os.path.basename(unaligned_sequence_database))
            shutil.copyfile(unaligned_sequence_database, unaligned_sequence_database_in_gpkg)
        
        if diamond_database_file:
            diamond_database_file_in_gpkg = os.path.basename(diamond_database_file)
            shutil.copyfile(diamond_database_file, os.path.join(output_package_path, diamond_database_file_in_gpkg))
        else:
            diamond_database_file_in_gpkg = diamond_database_file
        refpkg_in_gpkg = os.path.basename(refpkg_path)
        shutil.copytree(refpkg_path, os.path.join(output_package_path, refpkg_in_gpkg))
        
        if search_hmm_files:
            search_hmm_files_in_gpkg_names =\
                [os.path.basename(path) for path in search_hmm_files]
            if hmm_file_in_gpkg in search_hmm_files_in_gpkg_names:
                raise Exception("Search and align HMMs must all have different basenames to create a new gpkg")
            if len(set(search_hmm_files_in_gpkg_names)) != len(search_hmm_files_in_gpkg_names):
                raise Exception("Search HMMs must have different basenames to create a new gpkg")
            for i, search_hmm in enumerate(search_hmm_files):
                shutil.copyfile(search_hmm, os.path.join(output_package_path, search_hmm_files_in_gpkg_names[i]))
        else:
            search_hmm_files_in_gpkg_names = [hmm_file_in_gpkg]
        
        contents = {GraftMPackage.VERSION_KEY: GraftMPackageVersion3.version,
                    GraftMPackage.ALIGNMENT_HMM_KEY: hmm_file_in_gpkg,
                    GraftMPackage.SEARCH_HMM_KEY: search_hmm_files_in_gpkg_names,
                    GraftMPackage.REFERENCE_PACKAGE_KEY: refpkg_in_gpkg,
                    GraftMPackage.HMM_TRUSTED_CUTOFF_KEY: trusted_cutoff,
                    GraftMPackage.RANGE_KEY: max_range,
                    GraftMPackage.UNALIGNED_SEQUENCE_DATABASE_KEY: (os.path.basename(unaligned_sequence_database)
                                                                    if unaligned_sequence_database else None),
                    GraftMPackage.DIAMOND_DATABASE_KEY: diamond_database_file_in_gpkg}
        
        json.dump(contents, open(os.path.join(output_package_path, GraftMPackage._CONTENTS_FILE_NAME), 'w'))



################################################################################
################################# - Imports - ##################################

# Local imports
from graftm.getaxnseq import Getaxnseq
from graftm.rerooter import Rerooter
from graftm.greengenes_taxonomy import GreenGenesTaxonomy
# System imports
from skbio import TreeNode
import argparse
import logging
import os
import sys

################################################################################
################################ - Statics - ###################################
MAX_RESOLUTION = 7

################################################################################
################################## - Code - ####################################

class TreeUnrootedException(Exception): 
    pass

class TreeDecorator:
    '''Yet another class that decorates trees, only the way I want it.'''
    
    
    def __init__(self, 
                 tree,
                 taxonomy,
                 seqinfo=None):
        '''
        Set up empty lists and open and import tree object, and load the
        taxonomy into a hash
        '''
        
        self.gg_prefixes = ["k__", "d__", 'p__', 'c__', 'o__', 'f__', 'g__', 's__']
        
        # Set empty list to record nodes that have already been assigned
        # taxonomy.
        self.encountered_nodes = {}
        self.encountered_taxonomies = set()
        
        # Read in tree
        self.tree = tree
        
        # Read in taxonomy
        
        if seqinfo:
            gtns = Getaxnseq()
            logging.info("Importing taxtastic taxonomy from files: %s and %s" % (taxonomy, seqinfo))
            self.taxonomy =  gtns.read_taxtastic_taxonomy_and_seqinfo(open(taxonomy), open(seqinfo))
            
            for id, taxonomy_list in self.taxonomy.iteritems():
                if len(taxonomy_list) != MAX_RESOLUTION:
                    self.taxonomy[id] = taxonomy_list + self.gg_prefixes[1:][len(taxonomy_list):]
        else:
            logging.info("Importing greengenes taxonomy from file: %s" % (taxonomy))
            self.taxonomy = GreenGenesTaxonomy.read_file(taxonomy).taxonomy
            
    def _nodes(self):
        '''
        Iterate through nodes in tree, returning those that are not tips (i.e.
        returning only nodes)
        
        Inputs:
            None
        Yields:
            Node object, if it is not a tip
        '''
        logging.debug("Iterating through nodes")       
        if len(self.tree.children) > 2:
            logging.warning("There are > 2 descendants from the root of the \
tree that was  provided. Tree is being rerooted to the branch of the node that \
is the greatest distance to the root.")
            self.tree = Rerooter().reroot(self.tree)
        for node in self.tree.preorder():
            if node.is_root():
                logging.debug("%i reads in tree from root" % (len(list(node.tips()))))                   
                yield node
            elif node.is_tip():
                logging.debug("Tip %s reached" % (node.name))
            else:
                if node not in self.encountered_nodes: # If node hath not been decorated.
                    if node.name:
                        logging.debug("Node %s reached" % (node.name))
                    else:
                        logging.debug("Unnamed node reached")
                    yield node
    
    def _transpose_taxonomy(self, taxonomy_list):
        '''
        Transpose the taxonomy grid to contain lists of each taxonomic
        rank
        
        Inputs: 
            taxonomy_list: array
                - List of split taxonomy strings to be tansposed
        Outputs: 
            transposed_taxonomy: array
                - As per above description
        '''
        transposed_taxonomy = []
        for index in range(0, 7):

            rank_taxonomy_list = [taxonomy_entry[index] 
                                    for taxonomy_entry in taxonomy_list]
            transposed_taxonomy.append(set(rank_taxonomy_list))
        return transposed_taxonomy
    
    def _write_tree(self, output):
        '''Just Writes tree to file uisng scikit-bio'''
        logging.info("Writing decorated tree to file: %s" % (output))
        self.tree.write(
                    output,
                    format = "newick"   
                        )
        
    def _write_consensus_strings(self, output):
        '''Writes the taxonomy of each leaf to a file'''
        logging.info("Writing decorated taxonomy to file: %s" % (output))
        with open(output, 'w') as out:
            for tip in self.tree.tips():
                ancestors = tip.ancestors()
                
                ancestor_tax = []
                for ancestor in ancestors:
                    if ancestor.name:
                        split_node_name = ancestor.name.split(':')
                        if len(split_node_name) == 2:
                            ancestor_tax.append(split_node_name[1])
                        elif len(split_node_name) == 1:
                            pass
                        else:
                            raise Exception("Malformed node name: %s" % node.name)
                        
                tax_list = list(reversed(ancestor_tax))
                tax_name = tip.name.replace(" ", "_")
                if len(tax_list) < 1:
                    logging.debug("Species %s not decorated within the tree, using provided tax string as substitute" % (tax_name))
                    if tax_name in self.taxonomy:
                        tax_string = '; '.join(self.taxonomy[tax_name])
                    else:
                        logging.warning("No taxonomy found for species %s!" % (tax_name))
                        tax_string = "Unknown"
                else:
                    tax_string = '; '.join(tax_list)
                output_line = "%s\t%s\n" % (tax_name, tax_string)
                out.write(output_line)
    
    def _get_tax_index(self, ancestry):
        '''
        Iterate through ancestor nodes to find the current taxonomy index (i.e.
        the rank that should be assigned to the current node.
        
        Inputs:
            ancestry: list
                TreeNode.ancestory list that contains all nodes above the 
                current.
        Outputs:
            Current index of taxonomy. Returns None if node has no ancestors 
            (i.e. is root)
                
        '''
        current_index = 0
        ancetor_rank_number = 0
        ancestry = [x for x in ancestry if not x.is_root()]
        if any(ancestry):
            ancestor_tax_list = []
            for ancestor in ancestry:
                ai = self.encountered_nodes[ancestor]
                
                if ancestor.name:
                    split_node_name = ancestor.name.split(':')
                    if len(split_node_name) == 2:
                        ancestor_tax_list.append(split_node_name[1])
                    elif len(split_node_name) == 1:
                        pass
                    else:
                        raise Exception("Malformed node name: %s" % node.name)
                if ai > current_index:
                    current_index = ai
            
            ancestor_tax = '; '.join(reversed(ancestor_tax_list))
            logging.debug("Ancestors at %i nodes: %s" % (len(ancestor_tax_list), ancestor_tax))
            ancetor_rank_number = len(ancestor_tax.split('; ')) #
            return current_index, ancetor_rank_number
        else:       
            return current_index, ancetor_rank_number
            #raise Exception("Programming error in _get_tax_index. Failed to find ancestors for current node.")
    
    def extract(self, output_tax):
        self._write_consensus_strings(output_tax)
    
    def decorate(self, output_tree, output_tax, no_unique_names):
        '''
        Main function for TreeDecorator class. Accepts the output directory, 
        iterates through nodes, classifying them according to provided taxonomy. 
        '''
        # Define list of prefixes        
        logging.info("Decorating tree")
        for node in self._nodes():
            placement_depth = 0 
            current_index = 0
            parent = node.parent
            children = list(node.tips())
            logging.debug("Current node has %i children" % (len(children)))
                        
            # Gather the children 
            children_taxonomy = []
            for t in children:
                tip_name = t.name.replace(' ', '_')
                if tip_name in self.taxonomy:
                    children_taxonomy.append(self.taxonomy[tip_name])
            node_tax = self._transpose_taxonomy(children_taxonomy)
            tax_list = []
            for index, rank in enumerate(node_tax):
                if len(rank) == 1:
                    tax=list(rank)[0]
                    if tax not in self.gg_prefixes:
                        tax_list.append(list(rank)[0])
                        placement_depth += 1
                else:
                    break

            if len(tax_list) >= 1:
                logging.debug("Node has consistent taxonomy: %s" % '; '.join(tax_list))
                
                if not parent:
                    current_index = len(tax_list)
                    resolution = 1
                else:
                    if parent.is_root():
                        if parent.name:
                            current_index, resolution = self._get_tax_index(node.ancestors())
                        else:
                            current_index = 0
                            resolution = len(tax_list)   
                    else:
                        current_index, resolution = self._get_tax_index(node.ancestors())
                        
                    if len(tax_list) <= current_index:
                        logging.debug("No consistent taxonomy found for node! Moving to next node.")
                        self.encountered_nodes[node] = current_index
                        continue
                    else:
                        if len(tax_list) == placement_depth:

                            tax_list = tax_list[current_index:]
                            for child in children:
                                self.encountered_nodes[child] = current_index
                        elif len(node.children) == len(list(node.tips())):
                            tax_list = tax_list[current_index:]
                            for child in children:
                                self.encountered_nodes[child] = current_index
                        else:
                            current_index = len(tax_list)
                
                if resolution < MAX_RESOLUTION:
                    new_tax_list = [] 
                    
                        
                    for tax in tax_list:
                        if not no_unique_names:
                            if tax in self.encountered_taxonomies:
                                idx = 1
                                while tax in self.encountered_taxonomies:
                                    if idx > 1: 
                                        tax = '%s_%i' % ('_'.join(tax.split('_')[:-1]), idx)
                                    else:
                                        tax = tax + '_%i' % idx
                                    idx += 1

                        new_tax_list.append(tax)
                        self.encountered_taxonomies.add(tax)
                    tax_string = '; '.join(new_tax_list)
                    
                    if node.name:
                        split_node_name = node.name.split(':')
                        if len(split_node_name) > 2: 
                            raise Exception("Malformed tree. Please remove ':' from the read names in the tree")
                            exit(1)
                        elif len(split_node_name) < 2:
                            node_name = "%s:%s" % (split_node_name[0], tax_string)
                        else:
                            bootstrap_value, node_name = split_node_name[0], split_node_name[1]
                            node_name = "%s:%s" % (bootstrap_value, tax_string)
                    else:
                        node_name = tax_string
                    node.name = node_name
                    logging.debug("Renamed node %s" % node_name)
                    self.encountered_nodes[node]=index
                    
                else:
                    self.encountered_nodes[node]=current_index
            else:
                logging.debug("Cannot resolve node, no consistent taxonomy beyond that which has been described.")
                self.encountered_nodes[node]=current_index
            
        if output_tree:
            self._write_tree(output_tree)
        if output_tax:
            self._write_consensus_strings(output_tax)

################################################################################
################################################################################
        
        




class Deduplicator:
    r"""Deduplicates sequences"""
    
    def deduplicate(self, aligned_sequence_objects):
        r"""Sort the given aligned_sequence objects into an array of arrays,
        where input sequences are grouped iff they have the same sequence
        
        Parameters
        ----------
        aligned_sequence_objects: array of Sequence objects
            input sequences
            
        Returns
        -------
        Array of arrays of Sequence objects"""
        
        sequence_to_groups = {}
        for s in aligned_sequence_objects:
            try:
                sequence_to_groups[s.seq].append(s)
            except KeyError:
                sequence_to_groups[s.seq] = [s]
        return sequence_to_groups.values()
    
    def lca_taxonomy(self, deduplicated_sequences, taxonomy_hash):
        r'''Given a set of deduplicated sequences and a taxonomy hash,
        return the respective LCAs of taxonomy
        
        Parameters
        ----------
        deduplicated_sequences: Array of arrays of Sequence objects
            as output from deduplicate()
        taxonomy_hash: dictionary 
            of sequence names to taxonomy array (i.e. array of str)
        
        Returns
        -------
        Array of taxonomy LCAs'''
        
        to_return = []
        for dup_group in deduplicated_sequences:
            lca = taxonomy_hash[dup_group[0].name]
            for s in dup_group[1:]:
                for i, tax in enumerate(taxonomy_hash[s.name]):
                    if i >= len(lca) or tax != lca[i]:
                        lca = lca[:i]
                        break
                if len(lca) > len(taxonomy_hash[s.name]):
                    lca = lca[:len(taxonomy_hash[s.name])]
            to_return.append(lca)
        return to_return
                
            
        
        
        
        





class HMMreader:
    def __init__(self, table):
        self.entries = {}
        self.type    = [x.split()[2] for x in open(table, 'r') if x.startswith('# Program:')][0]
        table        = [x.rstrip().split() for x in open(table, 'r') if not x.startswith('#')]
        for entry in table:
            if entry[0] in self.entries:
                if self.type == "hmmsearch":    

                    if float(self.entries[entry[0]][15]) > float(entry[15]):
                        self.entries[entry[0]][15]=entry[15]
                    if float(self.entries[entry[0]][16]) < float(entry[16]):
                        self.entries[entry[0]][16]=entry[16]
                    
                elif self.type == "nhmmer":
                    if float(self.entries[entry[0]][13]) > float(entry[13]):
                        self.entries[entry[0]]=entry
            else:
                self.entries[entry[0]]=entry
        
    def names(self):
        return self.entries.keys()

    def evalue(self, entry):
        if self.type == "nhmmer":
            return float(self.entries[entry][12])
        elif self.type == "hmmsearch":
            return float(self.entries[entry][6])
        else: return None

    def name(self, entry):
        return self.entries[entry][0]

    def hmm_len(self, entry):
        if self.type == "nhmmer":
            return str(self.entries[entry][10])
        elif self.type == "hmmsearch":
            return str(self.entries[entry][5])
        else: return None

    def seq_len(self, entry):
        if self.type == "nhmmer":
            return str(self.entries[entry][10])
        elif self.type == "hmmsearch":
            return str(self.entries[entry][2])
        else: return None

    def hmmfrom(self, entry):
        if self.type == "nhmmer":
            return str(self.entries[entry][4])
        elif self.type == "hmmsearch":
            return str(self.entries[entry][15])
        else: return None

    def hmmto(self, entry):
        if self.type == "nhmmer":
            return str(self.entries[entry][5])
        elif self.type == "hmmsearch":
            return str(self.entries[entry][16])
        else: return None

    def aln_len(self, entry):
        if self.type == "nhmmer":
            tofrom=[float(self.entries[entry][5]),float(self.entries[entry][4])]
            return max(tofrom)-min(tofrom)
        elif self.type == "hmmsearch":
            tofrom=[float(self.entries[entry][15]),float(self.entries[entry][16])]
            return max(tofrom)-min(tofrom)
        else: return None

    def alifrom(self, entry):
        if self.type == "nhmmer":
            return str(self.entries[entry][6])
        elif self.type == "hmmsearch":
            return str(self.entries[entry][17])
        else: return None

    def alito(self, entry):
        if self.type == "nhmmer":
            return str(self.entries[entry][7])
        elif self.type == "hmmsearch":
            return str(self.entries[entry][18])
        else: return None

    def strand(self, entry):
        if self.type == "nhmmer":
            return self.entries[entry][11]
        elif self.type == "hmmsearch":
            return None
        else: return None

    def bit(self, entry):
        if self.type == "nhmmer":
            return float(self.entries[entry][13])
        elif self.type == "hmmsearch":
            return float(self.entries[entry][7])
        else: return None



#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import os.path
import sys
from io import StringIO
from skbio.tree import TreeNode

path_to_script = os.path.join(os.path.dirname(os.path.realpath(__file__)),'..','bin','graftM')
path_to_data = os.path.join(os.path.dirname(os.path.realpath(__file__)),'data')

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.taxonomy_extractor import TaxonomyExtractor

class Tests(unittest.TestCase):
    
    def test_hello_world(self):
        self.assertEquals({u'a': [],
                           u'b': [u't1'],
                           u'c': [u't1']},
                          TaxonomyExtractor().taxonomy_from_annotated_tree(\
                            TreeNode.read(StringIO(u"(a,(b,c)'t1':0.9)root;"))))

        

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import tempfile
import os
import sys
import io
from biom.util import biom_open
import subprocess

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.summarise import Stats_And_Summary

class Tests(unittest.TestCase):
    def test_iterate_otu_table_rows_hello_world(self):
        s = Stats_And_Summary()
        self.assertEqual(
                         [(1, ['ab','c'], [1])],
                         list(s._iterate_otu_table_rows([{'readname': ['ab','c']}]))
                         )  
        
    def test_iterate_otu_table_rows_two_samples_same_tax(self):
        s = Stats_And_Summary()
        self.assertEqual(
                         [(1, ['ab','c'], [1,1])],
                         list(s._iterate_otu_table_rows(({'readname': ['ab','c']},
                                                         {'readname2': ['ab','c']})))
                         )  
        
    def test_iterate_otu_table_rows_two_samples_different_counts(self):
        s = Stats_And_Summary()
        self.assertEqual(
                         [(1, ['ab','c'], [2,1])],
                         list(s._iterate_otu_table_rows([
                                                         {'readname': ['ab','c'], 'readname23': ['ab','c']},
                                                         {'readname2': ['ab','c']}
                                                         ]))
                         )  
        
    def test_iterate_otu_table_rows_two_samples_different_counts_twotax(self):
        s = Stats_And_Summary()
        self.assertEqual(
                         [(1, ['ab','d'], [1,0]), (2, ['ab','c'], [1,1])],
                         list(s._iterate_otu_table_rows([
                                                         {'readname': ['ab','c'], 'readname23': ['ab','d']},
                                                         {'readname2': ['ab','c']}
                                                         ]))
                         )
        
    def test_iterate_otu_table_rows_two_samples_second_new_tax(self):
        s = Stats_And_Summary()
        self.assertEqual(
                         [(1, ['ab','e'], [0,1]), (2, ['ab','c'], [2,0])],
                         list(s._iterate_otu_table_rows([
                                                         {'readname': ['ab','c'], 'readname23': ['ab','c']},
                                                         {'readname2': ['ab','e']}
                                                         ]))
                         )
        
    def test_write_otu_table(self):
        string = io.StringIO()
        s = Stats_And_Summary()
        s.write_tabular_otu_table(('sample1','sample2'),
                                  [
                                                         {'readname': ['ab','c']},
                                                         {'readname2': ['ab','c']}
                                                         ],
                                  string)
        self.assertEqual(u'#ID\tsample1\tsample2\tConsensusLineage\n1\t1\t1\tab; c\n', string.getvalue())
        
    def test_write_biom(self):
        with tempfile.NamedTemporaryFile(suffix='biom') as biom:
            with biom_open(biom.name,'w') as f:
                s = Stats_And_Summary()
                s.write_biom(('sample1','sample2'),
                          [
                                                 {'readname': ['ab','c'], 'readnameE': ['ab','d']},
                                                 {'readname2': ['ab','c']}
                                                 ],
                          f)

            with tempfile.NamedTemporaryFile(suffix='csv') as biom_out:
                os.remove(biom_out.name) #delete because otherwise biom complains
                subprocess.check_call("biom convert -i %s -o %s --table-type 'OTU table' --to-tsv --header-key taxonomy" %
                                                   (biom.name, biom_out.name), shell=True)
                observed = open(biom_out.name).read()
                self.assertEqual('''# Constructed from biom file
#OTU ID\tsample1\tsample2\ttaxonomy
1\t1.0\t0.0\tab; d
2\t1.0\t1.0\tab; c
''', observed)
        
        

if __name__ == "__main__":
    unittest.main()
